/*
 * $Id: nemesis-igmp.c,v 1.3 2000/01/16 10:31:54 obecian Exp $
 *
 * THE NEMESIS PROJECT (c) obecian 1999
 *
 * igmp.c (IGMP Packet Injector)
 *
 */

#include "nemesis-igmp.h"

int
main(int argc, char **argv)
{
	int             m;
	int             opt;
	char           *file = NULL;
	char            buff[512];
	FILE           *pay = NULL;
	extern char    *optarg;
	extern int      opterr;
    extern char    *nemesis_version;

	int             bindata = 0;

	got_link = 0;
	got_options = 0;
	got_payload = 0;

	verbose = 0;

	if (argc < 2)
		usage(argv[0]);

	if (geteuid() != 0) {
		printf("user '%s' does not have an euid of 0\n", getlogin());
		exit(1);
	}
	defaults();

	opterr = 0;
	while ((opt = getopt(argc, argv, "S:D:d:F:H:M:p:c:i:I:T:t:O:P:bv")) != EOF) {
		switch (opt) {
        case 'v':
            verbose = 1;
            putchar('\n');
            puts(TITLE " " VERSION);
            puts(CODERS);
            putchar('\n');
            break;
		case 'S':
			if (!(source = libnet_name_resolve(optarg, 0))) {
				fprintf(stderr, "Invalid source IP address: %s\n", optarg);
				exit(1);
			}
			if (verbose)
				printf("Source IP %s\n", optarg);
			break;
		case 'D':
			if (!(dest = libnet_name_resolve(optarg, 0))) {
				fprintf(stderr, "Invalid destination IP address: %s\n", optarg);
				exit(1);
			}
			if (verbose)
				printf("Destination IP %s\n", optarg);
			break;
        case 'i':
            if (!(iip = libnet_name_resolve(optarg, 0))) {
                fprintf(stderr, "Invalid IGMP group address: %s\n", optarg);
                exit(1);
            }
            if (verbose)
                printf("IGMP Group Address %s\n", optarg);
            break;
		case 'p': 
			type = atoi(optarg);
			break;
		case 'c': 
			code = atoi(optarg);
			break;
		case 'F':
			frag = atoi(optarg);
			break;
		case 'd':
			got_link = 1;
			device = optarg;
			break;
		case 'H':
			if (got_link)
            {
                sscanf(optarg, "%x:%x:%x:%x:%x:%x", &enet_tmp[0], &enet_tmp[1], &enet_tmp[2], &enet_tmp[3], &enet_tmp[4], &enet_tmp[5]);
                enet_src[0] = (u_char) enet_tmp[0];
                enet_src[1] = (u_char) enet_tmp[1];
                enet_src[2] = (u_char) enet_tmp[2];
                enet_src[3] = (u_char) enet_tmp[3];
                enet_src[4] = (u_char) enet_tmp[4];
                enet_src[5] = (u_char) enet_tmp[5];
            }
			break;
		case 'M':
			if (got_link)
            {
                sscanf(optarg, "%x:%x:%x:%x:%x:%x", &enet_tmp[0], &enet_tmp[1], &enet_tmp[2], &enet_tmp[3], &enet_tmp[4], &enet_tmp[5]);
                enet_dst[0] = (u_char) enet_tmp[0];
                enet_dst[1] = (u_char) enet_tmp[1];
                enet_dst[2] = (u_char) enet_tmp[2];
                enet_dst[3] = (u_char) enet_tmp[3];
                enet_dst[4] = (u_char) enet_tmp[4];
                enet_dst[5] = (u_char) enet_tmp[5];
            }
			break;
		case 'I':
			id = atoi(optarg);
			break;
		case 'T':
			ttl = atoi(optarg);
			break;
		case 't':
			tos = strtoul(optarg, NULL, 0);
			break;
		case 'b':
			bindata = 1;
			break;
		case 'P':
			file = optarg;
			got_payload = 1;
			if ((pay = fopen(file, "r")) == NULL) {
				printf("Payload file \"%s\"\n", file);
				exit(1);
			}
			break;
		case 'O':
			got_options = 1;
			(u_char) * options = strtoul(optarg, NULL, 0);
			break;
		case '?':
			usage(argv[0]);
			break;
		}
	}

	if (source == 0 || dest == 0) {
		printf("source and/or destination address missing.\n");
		exit(1);
	}
	if (verbose) {
		printf("IGMP Type: %d\n", type);
		printf("IGMP Code: %d\n", code);

		printf("IP ID: %d\n", id);
		printf("IP TTL: %d\n", ttl);
		printf("IP TOS: 0x%x\n", tos);
        printf("IP Frag: 0x%x\n", frag);
		printf("IP Options: %s\n\n", options);
	}
	if (got_payload) {
		if (bindata) {
			while (!feof(pay)) {
				fread(&buff, sizeof(buff), 1, pay);
				payload = malloc(sizeof(buff));
				strncpy(payload, buff, strlen(buff));
				if (verbose) {
					printf("BINARY Payload:\n");
					for (m = 0; m < strlen(payload); m++)
						printf("%x ", payload[m]);
					printf("\n");
				}
				if (buildigmp() == 0) {
					if (verbose)
						printf("\nWrote IGMP packet successfully\n");
					else if (verbose)
						printf("\nUnable to write packet successfully\n");
				}
			}
		} else {
			while (fgets(buff, sizeof(buff), pay)) {
				payload = malloc(sizeof(buff));
				strncpy(payload, buff, strlen(buff));
				if (verbose) {
					printf("ASCII Payload:\n");
					printf("%s\n", payload);
				}
				if (buildigmp() == 0) {
					if (verbose)
						printf("\nWrote IGMP packet successfully\n");
					else if (verbose)
						printf("\nUnable to write packet successfully\n");
				}
			}
		}
	} else {
		if (buildigmp() == 0) {
			if (verbose)
				printf("\nWrote IGMP packet successfully\n");
			else if (verbose)
				printf("\nUnable to write packet successfully\n");
		}
	}
}

void
usage(char *arg)
{
    putchar('\n');
    puts(TITLE " " VERSION);
    puts(CODERS);
    putchar('\n');
    
	printf("IGMP usage:\n  %s [-v] [options]\n\n", arg);
	printf("IGMP options: \n"
		   "  -p <IGMP Type>\n"
		   "  -c <IGMP Code>\n"
		   "  -i <IGMP Group Address>\n"
	       "  -P <Payload File (Binary or ASCII)>\n"
	       "  -b (Enable Binary Payload)\n"
	       "  (-v VERBOSE - packet struct to stdout)\n\n");
	printf("IP options: \n"
	       "  -S <Source IP Address>\n"
	       "  -D <Destination IP Address>\n"
	       "  -I <IP ID>\n"
	       "  -T <IP TTL>\n"
	       "  -t <IP tos>\n"
		   "  -F <IP frag>\n"
	       "  -O <IP Options>\n\n");
	printf("Data Link Options: \n"
	       "  -d <Ethernet Device>\n"
	       "  -H <Source MAC Address>\n"
	       "  -M <Destination MAC Address>\n\n");
	printf("You must define a Source, Destination and Protocol Options.\n");
	exit(1);
}

void
defaults()
{
	enet_src[0] = 0x02;
	enet_src[1] = 0x0f;
	enet_src[2] = 0x0a;
	enet_src[3] = 0x0d;
	enet_src[4] = 0x0e;
	enet_src[5] = 0x0d;

	enet_dst[0] = 0x0d;
	enet_dst[1] = 0x0e;
	enet_dst[2] = 0x0a;
	enet_dst[3] = 0x0d;
	enet_dst[4] = 0x00;
	enet_dst[5] = 0x01;

	type = 1;
	code = 1;
	iip = 0;
	id = 0;
	tos = IPTOS_LOWDELAY | IPTOS_THROUGHPUT;
	ttl = 1;
	payload = NULL;
	payload_s = 0;
	*options = NULL;
	option_s = 0;
	frag = IP_DF;
}
